@@ -256,7 +256,7 @@ setMethod("summary", "H2OModel", function(object, ...) {
   if( !is.null(tm$Gini)                                            )  cat("\nGini: (Extract with `h2o.gini`)", tm$Gini)
   if( !is.null(tm$null_deviance)                                   )  cat("\nNull Deviance: (Extract with `h2o.nulldeviance`)", tm$null_deviance)
   if( !is.null(tm$residual_deviance)                               )  cat("\nResidual Deviance: (Extract with `h2o.residual_deviance`)", tm$residual_deviance)
-  if(!is.null(o@algorithm) && o@algorithm %in% c("gam","glm","gbm","drf","xgboost","generic")) {
+  if(!is.null(o@algorithm) && o@algorithm %in% c("gam","glm","gbm","drf","xgboost","infogram","generic")) {
     if( !is.null(tm$r2) && !is.na(tm$r2)                           )  cat("\nR^2: (Extract with `h2o.r2`)", tm$r2)
   }
   if( !is.null(tm$AIC)                                             )  cat("\nAIC: (Extract with `h2o.aic`)", tm$AIC)
@@ -298,6 +298,9 @@ setClass("H2OUnknownModel",     contains="H2OModel")
 setClass("H2OBinomialModel",    contains="H2OModel")
 #' @rdname H2OModel-class
 #' @export
+setClass("H2OBinomialUpliftModel",    contains="H2OModel")
+#' @rdname H2OModel-class
+#' @export
 setClass("H2OMultinomialModel", contains="H2OModel")
 #' @rdname H2OModel-class
 #' @export
@@ -610,7 +613,7 @@ setMethod("show", "H2OBinomialMetrics", function(object) {
     cat("AUC:  ", object@metrics$AUC, "\n", sep="")
     cat("AUCPR:  ", object@metrics$pr_auc, "\n", sep="")
     cat("Gini:  ", object@metrics$Gini, "\n", sep="")
-    if(!is.null(object@algorithm) && object@algorithm %in% c("gam","glm","gbm","drf","xgboost","generic")) {
+    if(!is.null(object@algorithm) && object@algorithm %in% c("gam","glm","gbm","drf","xgboost","infogram","generic")) {
 
       if (!is.null(object@metrics$r2) && !is.na(object@metrics$r2)) cat("R^2:  ", object@metrics$r2, "\n", sep="")
       if (!is.null(object@metrics$null_deviance0)) cat("Null Deviance:  ", object@metrics$null_deviance,"\n", sep="")
@@ -639,6 +642,18 @@ setMethod("show", "H2OBinomialMetrics", function(object) {
     }
 })
 
+#' @rdname H2OModelMetrics-class
+#' @export
+setClass("H2OBinomialUpliftMetrics",    contains="H2OModelMetrics")
+#' @rdname H2OModelMetrics-class
+#' @export
+setMethod("show", "H2OBinomialUpliftMetrics", function(object) {
+    callNextMethod(object)  # call to the super
+    cat("Default AUUC:  ", object@metrics$AUUC, "\n", sep="")
+    cat("All types of AUUC:  ", "\n", sep="")
+    print(object@metrics$auuc_table)
+})
+
 #' @rdname H2OModelMetrics-class
 #' @export
 setClass("H2OMultinomialMetrics", contains="H2OModelMetrics")
@@ -730,7 +745,11 @@ setMethod("show", "H2OClusteringMetrics", function(object) {
     cat("\nTotal Within SS: ", m$tot_withinss)
     cat("\nBetween SS: ", m$betweenss)
     cat("\nTotal SS: ", m$totss, "\n")
-    print(m$centroid_stats)
+    if( !is.null(m$centroid_stats) ){
+      print(m$centroid_stats)
+    } else {
+      cat("Centroid statistics are not available.")
+    }
   } else print(NULL)
 })
 
@@ -920,6 +939,116 @@ setClass("H2OFrame", contains = c("Keyed", "environment"))
 #' @rdname h2o.keyof
 setMethod("h2o.keyof", signature("H2OFrame"), function(object) attr(object, "id"))
 
+setClassUnion("CharacterOrNULL", c("character", "NULL"))
+setClassUnion("numericOrNULL", c("numeric", "NULL"))
+setClassUnion("H2OFrameOrNULL", c("H2OFrame", "NULL"))
+
+#' H2OInfogram class
+#'
+#' H2OInfogram class contains a subset of what a normal H2OModel will return
+#' @slot model_id string returned as part of every \code{H2OModel}
+#' @slot algorithm string denoting the algorithm used to build infogram
+#' @slot admissible_features string array denoting all predictor names which pass the cmi and relelvance threshold
+#' @slot admissible_features_valid string array denoting all predictor names which pass the cmi and relelvance threshold from validation frame
+#' @slot admissible_features_xval string array denoting all predictor names which pass the cmi and relelvance threshold from cv holdout set
+#' @slot net_information_threshold numeric value denoting threshold used for predictor selection
+#' @slot total_information_threshold numeric value denoting threshold used for predictor selection
+#' @slot safety_index_threshold numeric value denoting threshold used for predictor selection
+#' @slot relevance_index_threshold numeric value denoting threshold used for predictor selection
+#' @slot admissible_score \code{H2OFrame} that contains columns, admissible, admissible_index, relevance, cmi, cmi_raw
+#' @slot admissible_score_valid \code{H2OFrame} that contains columns, admissible, admissible_index, relevance, cmi, cmi_raw from validation frame
+#' @slot admissible_score_xval \code{H2OFrame} that contains averages of columns, admissible, admissible_index, relevance, cmi, cmi_raw from cv hold-out
+#' @export
+setClass("H2OInfogram", slots = c(model_id = "character", 
+                                  algorithm = "character", 
+                                  admissible_features = "CharacterOrNULL",  
+                                  admissible_features_valid = "CharacterOrNULL", 
+                                  admissible_features_xval = "CharacterOrNULL",
+                                  net_information_threshold = "numericOrNULL", 
+                                  total_information_threshold = "numericOrNULL", 
+                                  safety_index_threshold = "numericOrNULL", 
+                                  relevance_index_threshold = "numericOrNULL", 
+                                  admissible_score = "H2OFrame", 
+                                  admissible_score_valid = "H2OFrameOrNULL", 
+                                  admissible_score_xval = "H2OFrameOrNULL"))
+
+#' Method on \code{H2OInfogram} object which in this case is to instantiate and initialize it
+#'
+#' @param .Object An \code{H2OInfogram} object
+#' @param model_id string returned as part of every H2OModel
+#' @param ... additional arguments to pass on
+#' @return A \code{H2OInfogram} object
+#' @export
+setMethod("initialize", "H2OInfogram", function(.Object, model_id, ...) {
+  if (!missing(model_id)) {
+    infogram_model <- h2o.getModel(model_id)
+    if (is(infogram_model, "H2OModel") &&
+        (infogram_model@algorithm == "infogram")) {
+      .Object@model_id <- infogram_model@model_id
+      .Object@algorithm <- infogram_model@algorithm
+      if (!is.null(infogram_model@model$admissible_features) && !is.list(infogram_model@model$admissible_features)) {
+        .Object@admissible_features <-
+        infogram_model@model$admissible_features
+      }
+      .Object@net_information_threshold <- infogram_model@parameters$net_information_threshold
+      .Object@total_information_threshold <- infogram_model@parameters$total_information_threshold 
+      .Object@safety_index_threshold <- infogram_model@parameters$safety_index_threshold
+      .Object@relevance_index_threshold <- infogram_model@parameters$relevance_index_threshold
+      .Object@admissible_score <- h2o.getFrame(infogram_model@model$admissible_score_key$name)
+      .Object@net_information_threshold <-
+        infogram_model@parameters$net_information_threshold
+      .Object@total_information_threshold <-
+        infogram_model@parameters$total_information_threshold
+      .Object@safety_index_threshold <-
+        infogram_model@parameters$safety_index_threshold
+      .Object@relevance_index_threshold <-
+        infogram_model@parameters$relevance_index_threshold
+      .Object@admissible_score <-
+        h2o.getFrame(infogram_model@model$admissible_score_key$name)
+      if (!is.null(infogram_model@model$admissible_features_valid) &&
+          !is.list(infogram_model@model$admissible_features_valid)) {
+        .Object@admissible_features_valid <-
+          infogram_model@model$admissible_features_valid
+      } else {
+        .Object@admissible_features_valid <- NULL
+      }
+      if (!is.null(infogram_model@model$admissible_features_xval) &&
+          !is.list(infogram_model@model$admissible_features_xval)) {
+        .Object@admissible_features_xval <-
+          infogram_model@model$admissible_features_xval
+      } else {
+        .Object@admissible_features_xval <- NULL
+      }
+      if (!is.null(infogram_model@model$admissible_score_key_valid)) {
+        .Object@admissible_score_valid <-
+          h2o.getFrame(infogram_model@model$admissible_score_key_valid$name)
+      } else {
+        .Object@admissible_score_valid <- NULL
+      }
+      if (!is.null(infogram_model@model$admissible_score_key_xval)) {
+        .Object@admissible_score_xval <-
+          h2o.getFrame(infogram_model@model$admissible_score_key_xval$name)
+      } else {
+        .Object@admissible_score_xval <- NULL
+      }
+      return(.Object)
+    } else {
+      stop('Input must be H2OModel with algorithm == "infogram".')
+    }
+  } else {
+    stop("A model Id must be used to instantiate a H2OInfogram.")
+  }
+})
+
+#' wrapper function for instantiating H2OInfogram
+#' @param model_id is string of H2OModel object
+#' @param ... parameters to algorithm, admissible_features, ...
+#' @return A \code{H2OInfogram} object
+#' @export
+H2OInfogram <- function(model_id, ...) {
+  initialize(new("H2OInfogram"), model_id = model_id, ...)
+}
+
 #'
 #' The H2OAutoML class
 #'
@@ -1081,4 +1210,4 @@ setMethod("h2o.varimp", signature("H2OFrame"), function(object) {
     stop("This is not a leaderboard frame. Only frames containing `model_id` column are supported.")
   }
   .varimp_matrix(object)
-})
\ No newline at end of file
+})
